"use strict";

const http = require('http');
const https = require('https');
const url = require('url');
const _ = require('lodash');


class Fetch {

	constructor(){
		this.config = {
			httpModule: http,
			httpsModule: https
		};
		return this;
	}

	/** http(s) request promissification
	 * @param {Object} options
	 * @returns {Promise}
	 * @private
	 */

	_request(options){
		const that = this;
		const _url = url.parse(options.url ? options.url : options);
		const _protocol = (_url.protocol ? _url.protocol.slice(0,-1) : 'http');

		const defaultRequestOptions = {
			hostname: _url.hostname,
			path: _url.path,
			encoding: 'utf8',
			method: options.method
		};

		return new Promise(function(done, error){

			const req = that.config[_protocol+'Module'].request(defaultRequestOptions, function(res) {

				let result = '';
				res.setEncoding(defaultRequestOptions.encoding);

				res.on('data', function (data) {
					result += data;
				});

				res.on('end', function(){
					done(result);
				});
			});

			req.on('error', function(err) {
				error(err);
			});

			req.end();

		});
	}



	/** GET method
	 *
	 * @param {string} url
	 * @param {Object} options
	 * @returns {Promise}
	 *
	 * */
	GET(url, options){
		const defaultOptions = {
			method: 'GET',
			url: url
		};
		const methodOptions =_.assign(defaultOptions, options);

		return this._request(methodOptions);
	}



}

module.exports = new Fetch();